@@ -116,11 +116,7 @@ class ListenersController(base.BaseController):
         if not db_pool:
             raise exceptions.NotFound(
                 resource=data_models.Pool._name(), id=pool_id)
-        if (db_pool.protocol == constants.PROTOCOL_UDP and
-                db_pool.protocol != listener_protocol):
-            msg = _("Listeners of type %s can only have pools of "
-                    "type UDP.") % constants.PROTOCOL_UDP
-            raise exceptions.ValidationException(detail=msg)
+        self._validate_protocol(listener_protocol, db_pool.protocol)
 
     def _has_tls_container_refs(self, listener_dict):
         return (listener_dict.get('tls_certificate_id') or
@@ -544,15 +540,16 @@ class ListenersController(base.BaseController):
                 driver_utils.listener_dict_to_provider_dict(listener_dict))
 
             # Also prepare the baseline object data
-            old_provider_llistener = (
-                driver_utils.db_listener_to_provider_listener(db_listener))
+            old_provider_listener = (
+                driver_utils.db_listener_to_provider_listener(db_listener,
+                                                              for_delete=True))
 
             # Dispatch to the driver
             LOG.info("Sending update Listener %s to provider %s", id,
                      driver.name)
             driver_utils.call_provider(
                 driver.name, driver.listener_update,
-                old_provider_llistener,
+                old_provider_listener,
                 driver_dm.Listener.from_dict(provider_listener_dict))
 
             # Update the database to reflect what the driver just accepted
