@@ -204,6 +204,7 @@ class PoolsController(base.BaseController):
         elif pool.listener_id:
             listener = self.repositories.listener.get(
                 context.session, id=pool.listener_id)
+            self._validate_protocol(listener.protocol, pool.protocol)
             pool.loadbalancer_id = listener.load_balancer_id
             pool.project_id, provider = self._get_lb_project_id_provider(
                 context.session, pool.loadbalancer_id)
@@ -403,7 +404,7 @@ class PoolsController(base.BaseController):
 
             # Also prepare the baseline object data
             old_provider_pool = driver_utils.db_pool_to_provider_pool(
-                db_pool)
+                db_pool, for_delete=True)
 
             # Dispatch to the driver
             LOG.info("Sending update Pool %s to provider %s", id, driver.name)
