@@ -1,5 +1,3 @@
-#!/usr/bin/env python
-
 #    Copyright 2014 Rackspace
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -22,8 +20,7 @@ from oslo_log import log as logging
 from oslo_reports import guru_meditation_report as gmr
 
 from octavia.api import app as api_app
-from octavia.common import service as octavia_service
-from octavia.i18n import _LI
+from octavia.common import constants
 from octavia import version
 
 
@@ -31,15 +28,19 @@ LOG = logging.getLogger(__name__)
 
 
 def main():
-    octavia_service.prepare_service(sys.argv)
-
     gmr.TextGuruMeditation.setup_autorun(version)
 
-    app = api_app.setup_app()
+    app = api_app.setup_app(argv=sys.argv)
 
-    host, port = cfg.CONF.bind_host, cfg.CONF.bind_port
-    LOG.info(_LI("Starting API server on %(host)s:%(port)s"),
+    host = cfg.CONF.api_settings.bind_host
+    port = cfg.CONF.api_settings.bind_port
+    LOG.info("Starting API server on %(host)s:%(port)s",
              {"host": host, "port": port})
+    if cfg.CONF.api_settings.auth_strategy != constants.KEYSTONE:
+        LOG.warning('Octavia configuration [api_settings] auth_strategy is '
+                    'not set to "keystone". This is not a normal '
+                    'configuration and you may get "Missing project ID" '
+                    'errors from API calls."')
     srv = simple_server.make_server(host, port, app)
 
     srv.serve_forever()
