@@ -18,9 +18,11 @@ Routines for configuring Octavia
 """
 
 import os
+import ssl
 import sys
 
 from keystoneauth1 import loading as ks_loading
+from octavia_lib.common import constants as lib_consts
 from oslo_config import cfg
 from oslo_db import options as db_options
 from oslo_log import log as logging
@@ -34,6 +36,9 @@ from octavia import version
 
 LOG = logging.getLogger(__name__)
 
+TLS_PROTOCOL_CHOICES = [
+    p[9:].replace('_', '.') for p in ssl._PROTOCOL_NAMES.values()]
+
 
 core_opts = [
     cfg.HostnameOpt('host', default=utils.get_hostname(),
@@ -52,8 +57,6 @@ api_opts = [
                         constants.KEYSTONE,
                         constants.TESTING],
                help=_("The auth strategy for API requests.")),
-    cfg.StrOpt('api_handler', default='queue_producer',
-               help=_("The handler that the API communicates with")),
     cfg.BoolOpt('allow_pagination', default=True,
                 help=_("Allow the usage of pagination")),
     cfg.BoolOpt('allow_sorting', default=True,
@@ -71,21 +74,18 @@ api_opts = [
                help=_("Base URI for the API for use in pagination links. "
                       "This will be autodetected from the request if not "
                       "overridden here.")),
-    cfg.BoolOpt('api_v1_enabled', default=True,
-                help=_("Expose the v1 API?")),
-    cfg.BoolOpt('api_v2_enabled', default=True,
-                help=_("Expose the v2 API?")),
     cfg.BoolOpt('allow_tls_terminated_listeners', default=True,
                 help=_("Allow users to create TLS Terminated listeners?")),
     cfg.BoolOpt('allow_ping_health_monitors', default=True,
                 help=_("Allow users to create PING type Health Monitors?")),
     cfg.DictOpt('enabled_provider_drivers',
-                help=_('List of enabled provider drivers and description '
-                       'dictionaries. Must match the driver name in the '
+                help=_('A comma separated list of dictionaries of the '
+                       'enabled provider driver names and descriptions. '
+                       'Must match the driver name in the '
                        'octavia.api.drivers entrypoint. Example: '
-                       '{\'amphora\': \'The Octavia Amphora driver.\', '
-                       '\'octavia\': \'Deprecated alias of the Octavia '
-                       'Amphora driver.\'}'),
+                       'amphora:The Octavia Amphora driver.,'
+                       'octavia:Deprecated alias of the Octavia '
+                       'Amphora driver.'),
                 default={'amphora': 'The Octavia Amphora driver.',
                          'octavia': 'Deprecated alias of the Octavia Amphora '
                          'driver.'}),
@@ -115,6 +115,54 @@ amphora_agent_opts = [
     cfg.IntOpt('agent_request_read_timeout', default=180,
                help=_("The time in seconds to allow a request from the "
                       "controller to run before terminating the socket.")),
+    cfg.StrOpt('agent_tls_protocol', default='TLSv1.2',
+               help=_("Minimum TLS protocol for communication with the "
+                      "amphora agent."),
+               choices=TLS_PROTOCOL_CHOICES),
+
+    # Logging setup
+    cfg.ListOpt('admin_log_targets',
+                help=_('List of log server ip and port pairs for '
+                       'Administrative logs. Additional hosts are backup to '
+                       'the primary server. If none is '
+                       'specified remote logging is disabled. Example '
+                       '127.0.0.1:10514, 192.168.0.1:10514')),
+    cfg.ListOpt('tenant_log_targets',
+                help=_('List of log server ip and port pairs for '
+                       'tenant traffic logs. Additional hosts are backup to '
+                       'the primary server. If none is '
+                       'specified remote logging is disabled. Example '
+                       '127.0.0.1:10514, 192.168.0.1:10514')),
+    cfg.IntOpt('user_log_facility', default=0, min=0, max=7,
+               help=_('LOG_LOCAL facility number to use for user traffic '
+                      'logs.')),
+    cfg.IntOpt('administrative_log_facility', default=1, min=0, max=7,
+               help=_('LOG_LOCAL facility number to use for amphora processes '
+                      'logs.')),
+    cfg.StrOpt('log_protocol', default=lib_consts.PROTOCOL_UDP,
+               choices=[lib_consts.PROTOCOL_TCP, lib_consts.PROTOCOL_UDP],
+               help=_("The log forwarding transport protocol. One of UDP or "
+                      "TCP.")),
+    cfg.IntOpt('log_retry_count', default=5,
+               help=_('The maximum attempts to retry connecting to the '
+                      'logging host.')),
+    cfg.IntOpt('log_retry_interval', default=2,
+               help=_('The time, in seconds, to wait between retries '
+                      'connecting to the logging host.')),
+    cfg.IntOpt('log_queue_size', default=10000,
+               help=_('The queue size (messages) to buffer log messages.')),
+    cfg.StrOpt('logging_template_override',
+               help=_('Custom logging configuration template.')),
+    cfg.BoolOpt('forward_all_logs', default=False,
+                help=_('When True, the amphora will forward all of the '
+                       'system logs (except tenant traffic logs) to the '
+                       'admin log target(s). When False, '
+                       'only amphora specific admin logs will be forwarded.')),
+    cfg.BoolOpt('disable_local_log_storage', default=False,
+                help=_('When True, no logs will be written to the amphora '
+                       'filesystem. When False, log files will be written to '
+                       'the local filesystem.')),
+
     # Do not specify in octavia.conf, loaded at runtime
     cfg.StrOpt('amphora_id', help=_("The amphora ID.")),
     cfg.StrOpt('amphora_udp_driver',
@@ -211,27 +259,10 @@ healthmanager_opts = [
                help=_('Driver for updating amphora health system.')),
     cfg.StrOpt('stats_update_driver', default='stats_db',
                help=_('Driver for updating amphora statistics.')),
-
-    # Used for synchronizing neutron-lbaas and octavia
-    cfg.StrOpt('event_streamer_driver',
-               help=_('Specifies which driver to use for the event_streamer '
-                      'for syncing the octavia and neutron_lbaas dbs. If you '
-                      'don\'t need to sync the database or are running '
-                      'octavia in stand alone mode use the '
-                      'noop_event_streamer'),
-               default='noop_event_streamer'),
-    cfg.BoolOpt('sync_provisioning_status', default=False,
-                help=_("Enable provisioning status sync with neutron db"))]
+]
 
 oslo_messaging_opts = [
     cfg.StrOpt('topic'),
-    cfg.StrOpt('event_stream_topic',
-               default='neutron_lbaas_event',
-               help=_('topic name for communicating events through a queue')),
-    cfg.StrOpt('event_stream_transport_url', default=None,
-               help=_('Transport URL to use for the neutron-lbaas '
-                      'synchronization event stream when neutron and octavia '
-                      'have separate queues.')),
 ]
 
 haproxy_amphora_opts = [
@@ -273,6 +304,10 @@ haproxy_amphora_opts = [
     cfg.StrOpt('haproxy_stick_size', default='10k',
                help=_('Size of the HAProxy stick table. Accepts k, m, g '
                       'suffixes.  Example: 10k')),
+    cfg.StrOpt('user_log_format',
+               default='{project_id} {lb_id} %f %ci %cp %t %{+Q}r %ST %B %U '
+                       '%[ssl_c_verify] %{+Q}[ssl_c_s_dn] %b %s %Tt %tsc',
+               help=_('Log format string for user flow logging.')),
 
     # REST server
     cfg.IPOpt('bind_host', default='::',  # nosec
@@ -381,6 +416,10 @@ controller_worker_opts = [
     cfg.StrOpt('network_driver',
                default='network_noop_driver',
                help=_('Name of the network driver to use')),
+    cfg.StrOpt('volume_driver',
+               default=constants.VOLUME_NOOP_DRIVER,
+               choices=constants.SUPPORTED_VOLUME_DRIVERS,
+               help=_('Name of the volume driver to use')),
     cfg.StrOpt('distributor_driver',
                default='distributor_noop_driver',
                help=_('Name of the distributor driver to use')),
@@ -400,8 +439,12 @@ controller_worker_opts = [
 
 task_flow_opts = [
     cfg.StrOpt('engine',
-               default='serial',
-               help=_('TaskFlow engine to use')),
+               default='parallel',
+               choices=constants.SUPPORTED_TASKFLOW_ENGINE_TYPES,
+               help=_('TaskFlow engine to use. '
+                      'serial - Runs all tasks on a single thread. '
+                      'parallel - Schedules tasks onto different threads to '
+                      'allow for running non-dependent tasks simultaneously')),
     cfg.IntOpt('max_workers',
                default=5,
                help=_('The maximum number of workers')),
@@ -471,17 +514,6 @@ house_keeping_opts = [
                       ' rotation'))
 ]
 
-anchor_opts = [
-    cfg.StrOpt('url',
-               default='http://localhost:9999/v1/sign/default',
-               help=_('Anchor URL')),
-    cfg.StrOpt('username',
-               help=_('Anchor username')),
-    cfg.StrOpt('password',
-               help=_('Anchor password'),
-               secret=True)
-]
-
 keepalived_vrrp_opts = [
     cfg.IntOpt('vrrp_advert_int',
                default=1,
@@ -526,8 +558,7 @@ nova_opts = [
                 help=_('Disable certificate validation on SSL connections')),
     cfg.BoolOpt('enable_anti_affinity', default=False,
                 help=_('Flag to indicate if nova anti-affinity feature is '
-                       'turned on. This option is only used when creating '
-                       'amphorae in ACTIVE_STANDBY topology.')),
+                       'turned on.')),
     cfg.StrOpt('anti_affinity_policy', default=constants.ANTI_AFFINITY,
                choices=[constants.ANTI_AFFINITY, constants.SOFT_ANTI_AFFINITY],
                help=_('Sets the anti-affinity policy for nova')),
@@ -539,6 +570,38 @@ nova_opts = [
     cfg.StrOpt('availability_zone', default=None,
                help=_('Availability zone to use for creating Amphorae')),
 ]
+
+cinder_opts = [
+    cfg.StrOpt('service_name',
+               help=_('The name of the cinder service in the keystone '
+                      'catalog')),
+    cfg.StrOpt('endpoint', help=_('A new endpoint to override the endpoint '
+                                  'in the keystone catalog.')),
+    cfg.StrOpt('region_name',
+               help=_('Region in Identity service catalog to use for '
+                      'communication with the OpenStack services.')),
+    cfg.StrOpt('endpoint_type', default='publicURL',
+               help=_('Endpoint interface in identity service to use')),
+    cfg.StrOpt('ca_certificates_file',
+               help=_('CA certificates file path')),
+    cfg.StrOpt('availability_zone', default=None,
+               help=_('Availability zone to use for creating Volume')),
+    cfg.BoolOpt('insecure',
+                default=False,
+                help=_('Disable certificate validation on SSL connections')),
+    cfg.IntOpt('volume_size', default=16,
+               help=_('Size of volume, in GB, for Amphora instance')),
+    cfg.StrOpt('volume_type', default=None,
+               help=_('Type of volume for Amphorae volume root disk')),
+    cfg.IntOpt('volume_create_retry_interval', default=5,
+               help=_('Interval time to wait volume is created in available'
+                      'state')),
+    cfg.IntOpt('volume_create_timeout', default=300,
+               help=_('Timeout to wait for volume creation success')),
+    cfg.IntOpt('volume_create_max_retries', default=5,
+               help=_('Maximum number of retries to create volume'))
+]
+
 neutron_opts = [
     cfg.StrOpt('service_name',
                help=_('The name of the neutron service in the '
@@ -616,6 +679,9 @@ driver_agent_opts = [
                default='/var/run/octavia/stats.sock',
                help=_('Path to the driver statistics unix domain socket '
                       'file.')),
+    cfg.StrOpt('get_socket_path',
+               default='/var/run/octavia/get.sock',
+               help=_('Path to the driver get unix domain socket file.')),
     cfg.IntOpt('status_request_timeout',
                default=5,
                help=_('Time, in seconds, to wait for a status update '
@@ -632,11 +698,25 @@ driver_agent_opts = [
                default=50,
                help=_('Maximum number of concurrent processes to use '
                       'servicing statistics updates.')),
+    cfg.IntOpt('get_request_timeout',
+               default=5,
+               help=_('Time, in seconds, to wait for a get request.')),
+    cfg.IntOpt('get_max_processes',
+               default=50,
+               help=_('Maximum number of concurrent processes to use '
+                      'servicing get requests.')),
     cfg.FloatOpt('max_process_warning_percent',
                  default=0.75, min=0.01, max=0.99,
                  help=_('Percentage of max_processes (both status and stats) '
                         'in use to start logging warning messages about an '
                         'overloaded driver-agent.')),
+    cfg.IntOpt('provider_agent_shutdown_timeout',
+               default=60,
+               help=_('The time, in seconds, to wait for provider agents '
+                      'to shutdown after the exit event has been set.')),
+    cfg.ListOpt('enabled_provider_agents', default='',
+                help=_('List of enabled provider agents. The driver-agent '
+                       'will launch these agents at startup.'))
 ]
 
 # Register the configuration options
@@ -650,11 +730,11 @@ cfg.CONF.register_opts(controller_worker_opts, group='controller_worker')
 cfg.CONF.register_opts(keepalived_vrrp_opts, group='keepalived_vrrp')
 cfg.CONF.register_opts(task_flow_opts, group='task_flow')
 cfg.CONF.register_opts(house_keeping_opts, group='house_keeping')
-cfg.CONF.register_opts(anchor_opts, group='anchor')
 cfg.CONF.register_cli_opts(core_cli_opts)
 cfg.CONF.register_opts(certificate_opts, group='certificates')
 cfg.CONF.register_cli_opts(healthmanager_opts, group='health_manager')
 cfg.CONF.register_opts(nova_opts, group='nova')
+cfg.CONF.register_opts(cinder_opts, group='cinder')
 cfg.CONF.register_opts(glance_opts, group='glance')
 cfg.CONF.register_opts(neutron_opts, group='neutron')
 cfg.CONF.register_opts(quota_opts, group='quotas')
@@ -683,7 +763,7 @@ def init(args, **kwargs):
              version='%%prog %s' % version.version_info.release_string(),
              **kwargs)
     handle_deprecation_compatibility()
-    setup_remote_pydev_debug()
+    setup_remote_debugger()
 
 
 def setup_logging(conf):
@@ -715,32 +795,54 @@ def handle_deprecation_compatibility():
                              group='health_manager')
 
 
-def setup_remote_pydev_debug():
+def _enable_pydev(debugger_host, debugger_port):
+    try:
+        from pydev import pydevd  # pylint: disable=import-outside-toplevel
+    except ImportError:
+        import pydevd  # pylint: disable=import-outside-toplevel
+
+    pydevd.settrace(debugger_host,
+                    port=int(debugger_port),
+                    stdoutToServer=True,
+                    stderrToServer=True)
+
+
+def _enable_ptvsd(debuggger_host, debugger_port):
+    import ptvsd  # pylint: disable=import-outside-toplevel
+
+    # Allow other computers to attach to ptvsd at this IP address and port.
+    ptvsd.enable_attach(address=(debuggger_host, debugger_port),
+                        redirect_output=True)
+
+    # Pause the program until a remote debugger is attached
+    ptvsd.wait_for_attach()
+
+
+def setup_remote_debugger():
     """Required setup for remote debugging."""
 
-    pydev_debug_host = os.environ.get('PYDEV_DEBUG_HOST')
-    pydev_debug_port = os.environ.get('PYDEV_DEBUG_PORT')
+    debugger_type = os.environ.get('DEBUGGER_TYPE', 'pydev')
+    debugger_host = os.environ.get('DEBUGGER_HOST')
+    debugger_port = os.environ.get('DEBUGGER_PORT')
 
-    if not pydev_debug_host or not pydev_debug_port:
+    if not debugger_type or not debugger_host or not debugger_port:
         return
 
     try:
-        try:
-            from pydev import pydevd
-        except ImportError:
-            import pydevd
-
         LOG.warning("Connecting to remote debugger. Once connected, resume "
                     "the program on the debugger to continue with the "
                     "initialization of the service.")
-        pydevd.settrace(pydev_debug_host,
-                        port=int(pydev_debug_port),
-                        stdoutToServer=True,
-                        stderrToServer=True)
+        if debugger_type == 'pydev':
+            _enable_pydev(debugger_host, debugger_port)
+        elif debugger_type == 'ptvsd':
+            _enable_ptvsd(debugger_host, debugger_port)
+        else:
+            LOG.exception('Debugger %(debugger)s is not supported',
+                          debugger_type)
     except Exception:
         LOG.exception('Unable to join debugger, please make sure that the '
                       'debugger processes is listening on debug-host '
                       '\'%(debug-host)s\' debug-port \'%(debug-port)s\'.',
-                      {'debug-host': pydev_debug_host,
-                       'debug-port': pydev_debug_port})
+                      {'debug-host': debugger_host,
+                       'debug-port': debugger_port})
         raise
