@@ -205,6 +205,14 @@ DOMAIN_NAME = 'domain_name'
 UPDATE_STATS = 'UPDATE_STATS'
 UPDATE_HEALTH = 'UPDATE_HEALTH'
 
+VALID_LISTENER_POOL_PROTOCOL_MAP = {
+    PROTOCOL_TCP: [PROTOCOL_HTTP, PROTOCOL_HTTPS,
+                   PROTOCOL_PROXY, PROTOCOL_TCP],
+    PROTOCOL_HTTP: [PROTOCOL_HTTP, PROTOCOL_PROXY],
+    PROTOCOL_HTTPS: [PROTOCOL_HTTPS, PROTOCOL_PROXY, PROTOCOL_TCP],
+    PROTOCOL_TERMINATED_HTTPS: [PROTOCOL_HTTP, PROTOCOL_PROXY],
+    PROTOCOL_UDP: [PROTOCOL_UDP]}
+
 # API Integer Ranges
 MIN_PORT_NUMBER = 1
 MAX_PORT_NUMBER = 65535
