@@ -95,8 +95,10 @@ class CreateAmphoraInDB(BaseDatabaseTask):
         :returns: The created amphora object
         """
 
+        loadbalancer_id = kwargs.get("loadbalancer_id", None)
         amphora = self.amphora_repo.create(db_apis.get_session(),
                                            id=uuidutils.generate_uuid(),
+                                           load_balancer_id=loadbalancer_id,
                                            status=constants.PENDING_CREATE,
                                            cert_busy=False)
 
@@ -522,9 +524,16 @@ class MapLoadbalancerToAmphora(BaseDatabaseTask):
                       "allocation.")
             return None
 
-        amp = self.amphora_repo.allocate_and_associate(
-            db_apis.get_session(),
-            loadbalancer_id)
+        try:
+            amp = self.amphora_repo.allocate_and_associate(
+                db_apis.get_session(),
+                loadbalancer_id)
+        except Exception as e:
+            LOG.error("Failed to get a spare amphora for "
+                      "loadbalancer {} due to: {}".format(
+                          loadbalancer_id, e))
+            return None
+
         if amp is None:
             LOG.debug("No Amphora available for load balancer with id %s",
                       loadbalancer_id)
@@ -1176,14 +1185,14 @@ class MarkLBAndListenersActiveInDB(BaseDatabaseTask):
         """
 
         LOG.debug("Mark ACTIVE in DB for load balancer id: %s "
-                  "and listener ids: %s", loadbalancer.id,
+                  "and updating status for listener ids: %s", loadbalancer.id,
                   ', '.join([l.id for l in listeners]))
         self.loadbalancer_repo.update(db_apis.get_session(),
                                       loadbalancer.id,
                                       provisioning_status=constants.ACTIVE)
         for listener in listeners:
-            self.listener_repo.update(db_apis.get_session(), listener.id,
-                                      provisioning_status=constants.ACTIVE)
+            self.listener_repo.prov_status_active_if_not_error(
+                db_apis.get_session(), listener.id)
 
     def revert(self, loadbalancer, listeners, *args, **kwargs):
         """Mark the load balancer and listeners as broken.
@@ -1202,35 +1211,6 @@ class MarkLBAndListenersActiveInDB(BaseDatabaseTask):
             self.task_utils.mark_listener_prov_status_error(listener.id)
 
 
-class MarkListenerActiveInDB(BaseDatabaseTask):
-    """Mark the listener active in the DB.
-
-    Since sqlalchemy will likely retry by itself always revert if it fails
-    """
-
-    def execute(self, listener):
-        """Mark the listener as active in DB
-
-        :param listener: The listener to be marked active
-        :returns: None
-        """
-
-        LOG.debug("Mark ACTIVE in DB for listener id: %s ", listener.id)
-        self.listener_repo.update(db_apis.get_session(), listener.id,
-                                  provisioning_status=constants.ACTIVE)
-
-    def revert(self, listener, *args, **kwargs):
-        """Mark the listener ERROR since the delete couldn't happen
-
-        :param listener: The listener that couldn't be updated
-        :returns: None
-        """
-
-        LOG.warning("Reverting mark listener active in DB "
-                    "for listener id %s", listener.id)
-        self.task_utils.mark_listener_prov_status_error(listener.id)
-
-
 class MarkListenerDeletedInDB(BaseDatabaseTask):
     """Mark the listener deleted in the DB.
 
